package com.syntevo.plugin.jira.commit;

import java.io.*;
import java.util.*;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.openapi.smartsvn.settings.*;
import com.syntevo.plugin.jira.*;
import com.syntevo.plugin.jira.transport.*;

/**
 * @author syntevo GmbH
 */
public final class JiraUIConnection implements JiraConnection {

	// Constants ==============================================================

	private static final Map<String, String> volatileSettings = new HashMap<String, String>();
	private static final String KEY_USERNAME = "username";
	private static final String KEY_PASSWORD = "password";
	private static final String KEY_CLIENT_CERTIFICATE = "client-certificate";
	private static final String KEY_CLIENT_CERTIFICATE_PASSPHRASE = "client-certificate-passphrase";
	private static final String KEY_SERVER_SHA_FINGERPRINT = "server-sha-fingerprint";
	private static final boolean DEBUG_LOGGING = Boolean.getBoolean("smartsvn.plugin.jira.debug-logging");

	// Static =================================================================

	private static PrintWriter log;

	// Fields =================================================================

	private final String url;
	private final SettingsServices settingsServices;
	private final DialogDisplayer dialogDisplayer;
	private final boolean silent;
	private final GuiSpacings spacings;

	// Setup ==================================================================

	public JiraUIConnection(String url, SettingsServices settingsServices, boolean silent, GuiSpacings spacings, DialogDisplayer dialogDisplayer) {
		this.settingsServices = settingsServices;
		this.dialogDisplayer = dialogDisplayer;
		this.silent = silent;
		this.url = url.endsWith("/") ? url : url + "/";
		this.spacings = spacings;
	}

	// Implemented ============================================================

	public String getUrl() {
		return url;
	}

	public String getUsername() {
		if (!checkJiraCredentials()) {
			return null;
		}

		return retrieve(url, KEY_USERNAME);
	}

	public String getPassword() {
		if (!checkJiraCredentials()) {
			return null;
		}

		return retrieve(url, KEY_PASSWORD);
	}

	public void acknowledgeJiraCredentials(boolean success) {
		final String username = success ? retrieve(url, KEY_USERNAME) : null;
		final String password = success ? retrieve(url, KEY_PASSWORD) : null;
		store(url, KEY_USERNAME, username, Boolean.TRUE);
		store(url, KEY_PASSWORD, password, password == null ? Boolean.FALSE : null);
	}

	public boolean checkSSLServerFingerprintTrusted(String fingerprintSHA, String fingerprintMD5, String url) {
		final String fingerprint = retrieve(this.url, KEY_SERVER_SHA_FINGERPRINT);
		if (fingerprint == null) {
			if (silent || !dialogDisplayer.showConfirmationDialog("JIRA", "Please verify the SSL certificate for " + url, "The fingerprints are:\n" + "SHA: " + fingerprintSHA + "\nMD5: " + fingerprintMD5, "Confirm", false)) {
				return false;
			}
		}
		else if (!fingerprint.equals(fingerprintSHA)) {
			if (silent || !dialogDisplayer.showConfirmationDialog("JIRA", "The SSL certificate for " + url + " has changed.", "If you are not sure whether this is OK, please contact the server administrator. The fingerprints are:\n" + "SHA: " + fingerprintSHA + "\nMD5: " + fingerprintMD5, "Confirm", false)) {
				return false;
			}
		}

		store(this.url, KEY_SERVER_SHA_FINGERPRINT, fingerprintSHA, Boolean.TRUE);
		return true;
	}

	public JiraHttpsClientCertificate getSSLClientCertificate(String location) {
		if (!checkSSLCredentials()) {
			return null;
		}

		final String certificate = retrieve(url, KEY_CLIENT_CERTIFICATE);
		final String certificatePassword = retrieve(url, KEY_CLIENT_CERTIFICATE_PASSPHRASE);
		return new JiraHttpsClientCertificate(new File(certificate), certificatePassword);
	}

	public void showError(String message, Throwable ex) {
		if (silent) {
			return;
		}

		JiraPlugin.LOGGER.error(message, ex);
		dialogDisplayer.showErrorDialog("JIRA", message, ex.getMessage());
	}

	public void acknowledgeSSLClientCertificate(boolean success) {
		final String certificate = success ? retrieve(url, KEY_CLIENT_CERTIFICATE) : null;
		final String certificatePassword = success ? retrieve(url, KEY_CLIENT_CERTIFICATE_PASSPHRASE) : null;
		store(url, KEY_CLIENT_CERTIFICATE, certificate, Boolean.TRUE);
		store(url, KEY_CLIENT_CERTIFICATE_PASSPHRASE, certificatePassword, certificatePassword == null ? Boolean.FALSE : null);
	}

	public PrintWriter getLog() throws IOException {
		synchronized (JiraUIConnection.class) {
			if (!DEBUG_LOGGING) {
				return null;
			}

			if (log != null) {
				return log;
			}

			log = new PrintWriter(new FileWriter(settingsServices.getSettingsFile("jira-plugin.log")));
			return log;
		}
	}

	// Utils ==================================================================

	private boolean checkJiraCredentials() {
		final String username = retrieve(url, KEY_USERNAME);
		final String password = retrieve(url, KEY_PASSWORD);
		if (username != null && password != null) {
			return true;
		}

		if (silent) {
			return false;
		}

		final boolean[] success = new boolean[1];
		GuiUtils.executeInEventDispatchThread(new Runnable() {
			public void run() {
				final JiraCredentialsDialog dialog = new JiraCredentialsDialog(username, password, password != null, spacings);
				if (!dialogDisplayer.show(dialog)) {
					return;
				}

				final Boolean storePassword = Boolean.valueOf(dialog.isStorePassword());
				store(url, KEY_USERNAME, dialog.getUsername(), storePassword);
				store(url, KEY_PASSWORD, dialog.getPassword(), storePassword);
				success[0] = true;
			}
		});
		return success[0];
	}

	private boolean checkSSLCredentials() {
		final String clientCertificate = retrieve(url, KEY_CLIENT_CERTIFICATE);
		final String clientCertificatePassword = retrieve(url, KEY_CLIENT_CERTIFICATE_PASSPHRASE);
		if (clientCertificate != null && clientCertificatePassword != null) {
			return true;
		}

		if (silent) {
			return false;
		}

		final boolean[] success = new boolean[1];
		GuiUtils.executeInEventDispatchThread(new Runnable() {
			public void run() {
				final JiraClientCertificateDialog dialog = new JiraClientCertificateDialog(clientCertificate != null ? new File(clientCertificate) : null, clientCertificatePassword, clientCertificatePassword != null, spacings);
				if (!dialogDisplayer.show(dialog)) {
					return;
				}

				final Boolean storePassphrase = Boolean.valueOf(dialog.isStorePassphrase());
				store(url, KEY_CLIENT_CERTIFICATE, dialog.getCertificate().getAbsolutePath(), storePassphrase);
				store(url, KEY_CLIENT_CERTIFICATE_PASSPHRASE, dialog.getPassphrase(), storePassphrase);
				success[0] = true;
			}
		});
		return success[0];
	}

	private void store(String baseUrl, String key, String value, Boolean permanentNotTemporary) {
		key = baseUrl + "-" + key;
		volatileSettings.put(key, value);

		final Settings settings = settingsServices.getGlobalSettingsProvider().getSettings(JiraPlugin.PLUGIN_ID);
		if (permanentNotTemporary != null && permanentNotTemporary.booleanValue()) {
			settings.putString(key, value);
		}
		else if (permanentNotTemporary != null && !permanentNotTemporary.booleanValue()) {
			settings.removeValue(key);
		}
	}

	private String retrieve(String baseUrl, String key) {
		key = baseUrl + "-" + key;
		String value = volatileSettings.get(key);
		if (value != null) {
			return value;
		}

		value = settingsServices.getGlobalSettingsProvider().getSettings(JiraPlugin.PLUGIN_ID).getString(key, null);
		if (value != null) {
			volatileSettings.put(key, value);
		}
		return value;
	}
}